import os
from six import iteritems, string_types
import iris
from iris.cube import CubeList
from iris.coords import AuxCoord
from iris.util import squeeze
import yaml
import numba

def create_mask(mesh_file, config_file=None):
    if not config_file:
        cwd = os.path.dirname(os.path.realpath(__file__))
        config_file = os.path.join(cwd, 'basins.yml')

    with open(config_file, 'r') as yfile:
        config = yaml.safe_load(yfile)

    sea_mask = squeeze(iris.load_cube(mesh_file, 'tmask'))[0, ...]
    lat = iris.load_cube(mesh_file, 'nav_lat').data
    lon = iris.load_cube(mesh_file, 'nav_lon').data
    basins = CubeList()

    basin_model = sea_mask.copy()


    print('\nCreating basins from boxes')
    for basin, basin_config in iteritems(config['basins']):
        print('Creating basin', basin)
        add_boxes = flatten_list(basin_config.get('add', []))
        add_mask =  merge_boxes(add_boxes, lat, lon)

        remove_boxes = flatten_list(basin_config.get('remove', []))
        remove_mask =  merge_boxes(remove_boxes, lat, lon)

        mask = add_mask
        if remove_mask is not None:
            mask = merge_mask(add_mask, remove_mask)

        basin_mask = basin_model.copy() * mask
        basins.append(prepare_cube(basin_mask, basin))


    print('\nCreating basins by merging others')
    for basin, basin_config in iteritems(config['combined']):
        print('Creating basin', basin, 'from', basin_config)

        mask = merge_basic_basins(
            basin_config.get('add', []),
            basins
        )
        remove_mask = merge_basic_basins(
            basin_config.get('remove', []),
            basins
        )
        if remove_mask is not None:
            mask = merge_mask(mask, normalize(remove_mask))

        basin_mask = basin_model.copy(mask)
        basins.append(prepare_cube(basin_mask, basin))

    print('Merging and saving')
    basins = basins.merge_cube()
    basins.add_aux_coord(
        AuxCoord(lat, 'latitude', 'nav_lat'),
        (1, 2)
    )
    basins.add_aux_coord(
        AuxCoord(lat, 'longitude', 'nav_lon'),
        (1, 2)
    )
    iris.save(basins,  'basins.nc', zlib=True)

    import matplotlib
    matplotlib.use('Qt5Agg')
    import matplotlib.pyplot as plt
    import iris.quickplot as qplt
    print(basins)
    basins.remove_coord('latitude')
    basins.remove_coord('longitude')
    for basin_slice in basins.slices_over('region'):
        basin_slice.long_name = basin_slice.coord('region').points[0]
        basin_slice.standard_name = None
        qplt.pcolormesh(basin_slice)
        plt.show()

def flatten_list(config, final_type=dict):
    final_list = []
    for conf in config:
        if isinstance(conf, dict) or isinstance(conf, string_types):
            final_list.append(conf)
        else:
            final_list.extend(conf)
    return final_list

def merge_boxes(box_list, lat, lon):
    if not box_list:
        return None

    mask = None
    for box in box_list:
        current_box = apply_config(box['lat'][0], box['lat'][1], lat) * apply_config(box['lon'][0], box['lon'][1], lon)
        if mask is None:
            mask = current_box
        else:
            mask += current_box
    return normalize(mask)

def merge_basic_basins(config, basins):
    seas = []
    for sea in flatten_list(config):
        seas.append(basins.extract(iris.Constraint(region=sea))[0])
    if not seas:
        return None

    mask = None
    for sea in seas:
        if mask is None:
            mask = sea.data.copy()
        else:
            mask += sea.data
    return normalize(mask)

def prepare_cube(cube, basin):
    cube.add_aux_coord(AuxCoord([basin], var_name='region'))
    cube.var_name = 'basin'
    cube.standard_name = 'region'
    cube.long_name = 'Regional masks'
    cube.attributes = {}
    return cube

@numba.vectorize()
def apply_config(min, max, data):
    if data < min:
        return 0
    if data > max:
        return 0
    return 1

@numba.vectorize()
def normalize(data):
    if data < 1:
        return 0
    return 1

@numba.vectorize()
def merge_mask(add, remove):
    if add - remove < 1:
        return 0
    return 1

@numba.vectorize()
def intersection(mask1, mask2):
    if mask1 and mask2:
        return 1
    return 0


if __name__ == "__main__":
    create_mask(
        '/Users/nube/cpg_tools/new_masks/mesh_mask_nemo.Ec3.2_O1L75.nc'
    )